/************************************************************************/
/*									*/
/*	PORTFNS.C	"Port Functions"				*/
/*									*/
/*   This file contains the functions that actually read and write	*/
/* bytes to and from the port.  These functions should be called when-	*/
/* ever communication through the port to the Initializer or CRYPTOCard	*/
/* is desired.  Also found herein is a function that initializes the	*/
/* settings of the port to that required by the Initializer.		*/
/*									*/
/*   (C) - 1991-1996 CRYPTOCard Corporation.  All rights reserved.	*/
/*									*/
/************************************************************************/

static char sccsid[] =
"@(#)portfns.c  v1.0 Copyright 1996 CRYPTOCard Corporation";

/* Header files:*/
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <a.out.h>
#include <stdio.h>
#include <errno.h>
#include <termios.h>
#include <string.h>
#include "intlzr.h"

extern int _PortFD;


/*
 * SendByte function:
 *   Send a single byte out the port.  Returns true if successful, false otherwise.
 */
int SendByte (char Byte )
	{
  	ssize_t num;
	char data;
	data = Byte;
	num = write( _PortFD, &data, (size_t)1 );
	return((num == 1) ? TRUE : FALSE);
	}


/*
 * GetByte Function:
 *   Reads a byte from the port.  Returns number of bytes read or <0 if failure
 */
int GetByte ( char *Byte )
	{
	ssize_t num;
	num = read( _PortFD, Byte, (size_t)1 );
	return (num);
	}

/*
 * SendByteToFixture function:
 *   Sends a single character to the initializer hardware by asserting RTS, sending
 * the byte, reading the status returned and clearing RTS.  Returns the 
 * hardware status if successful (usually 0xff or 0xfe) or 0 if failure occurs
 */
int SendByteToFixture (byte Byte)
	{
  	char Status;
	int state;

	/* set rts to talk to the initializer */
	state = TIOCM_RTS;
	if( ioctl( _PortFD, TIOCMBIS,  &state ) >= 0 ) {
		do { 
    			SendByte (Byte);		/* Send the Byte	*/
		} while (GetByte (&Status) <= 0);	/* Wait for response	*/
  	}
	else
		/* the RTS line is not set properly */
		return( 0 );
	/* got the status, now clear the RTS line */
	state = TIOCM_RTS;
	if( ioctl( _PortFD, TIOCMBIC,  &state ) >= 0 ) 
  		return (Status);	

	/* couldn't clear the RTS line, failure */
	return( 0 );
	}


int IsCardIn (void)
	{
  	if (SendByteToFixture (LED_INSERT) & 0x01)/* This return code	*/
    		return (TRUE);			/* shows it's there	*/
  	else					/* Otherwise,		*/
    		return (FALSE);			/* It ain't!		*/
	}



int GetBufFromCard (char *BufPtr, int Length)
	{
  	char Byte;
	int num;
  	for ( ; Length > 0; Length--, BufPtr++) 
		{/* Read Length bytes	*/
    		num = GetByte (&Byte);		/* Grab a byte		*/
    		if (num == 1)			/* If it got a byte	*/
      			*(BufPtr) = Byte;	/* Put it in the buffer	*/
    		else			
      			return (FALSE);		/* Can't touch that	*/
  		}				/* End the loop		*/
  	return (TRUE);				/* It got all the bytes	*/
	}



int SendOctal (oct Octal)
/************************************************************************/
/* This function sends the octal digits passed in the parameter out to	*/
/* the CRYPTOCard, appending the SCROLL_KEY to the end.  The octal 	*/
/* digits are in ASCII format (ie "134" for 134).  If they were sent	*/
/* successfully, this function returns TRUE, otherwise it returns	*/
/* FALSE.								*/
/************************************************************************/
  {
  int Counter=0;
  char data;
  
  for ( ; Counter<3; Counter++) 
    {/* 3 digits in octal 	*/
    if (!(SendByte (Octal[Counter] - 0x30)))  /* Send and check	*/
      return (FALSE);				/* if it went well	*/
    GetByte (&data);				/* Get byte number and	*/
    }						/* ignore it		*/
  
  if (!(SendByte (SCROLL_KEY)))		/* Send the scroll key	*/
    return (FALSE);
 
  GetByte (&data);
  return (TRUE);
  }



int SendDec (char * Decimal, int Length)
/************************************************************************/
/* This function sends the string of decimal numbers pointed to in the	*/
/* parameter across to the CRYPTOCard.  If there was no communication	*/
/* error, it returns TRUE, otherwise: FALSE.  The string is in ASCII	*/
/* format. Ie "13312459".  It is used for sending PIN's and the back-	*/
/* door reset sequence.  It sends out Length bytes from the string.	*/
/************************************************************************/
{
  for ( ; Length > 0; Length--)			/* Go until Length = 0	*/
    if (!(SendByte (*(Decimal++)-0x30)))	/* Send it and check	*/
      return (FALSE);				/* If error-tell caller	*/
  return (TRUE);				/* All were nicely sent	*/
}



int GotThisBuf (char * Buffer)
/************************************************************************/
/* This function reads a buffer from the CRYPTOCard and compares it to	*/
/* the buffer pointed to in the parameter.  If they are equal, it	*/
/* returns TRUE, otherwise it returns FALSE.				*/
/************************************************************************/
{
  char *CardBuffer, *BackUp;		/* Some overly-large buffer	*/
  int Equal = TRUE;			/* Optimistic default return	*/
  int Length = 0;			/* The length of the buffer	*/

  BackUp = Buffer;			/* Save a copy of the pointer	*/

  while (*(Buffer)++)			/* Calculate the length of the	*/
    Length ++;				/* NULL terminated buffer	*/

  Buffer = BackUp;			/* Restore the pointer		*/

  BackUp = CardBuffer = (char *) malloc (Length+1);	/* Grab a bufr	*/

  if (!(GetBufFromCard (CardBuffer, Length)))	/* Get the Buffer	*/
    Equal = FALSE;			/* Communication error		*/
  while (*Buffer)			/* Go until we hit a NULL	*/
    if (*(CardBuffer++) != *(Buffer++))	/* If there is a discrepancy	*/
      Equal = FALSE;			/* indicate this to caller	*/
  free (BackUp);			/* Free up those 41 bytes	*/
  return (Equal);			/* Return the calculated value	*/
}
	
/*
 * InitPort function:
 *   This function initializes the device init_port to be ready to
 * interface with the initializer.  If any error occurs along the way
 * the program will exit with a status of -1.  If all initialization
 * is successful, the original termios settings for the port will be 
 * returned along with the file descriptor for the open device.
 *
 */
void InitPort( char *init_port, struct termios *saved_term, int *sldisc, int *port_fd )
	{
	int ldisc;
	int saved_ldisc;
	int state;
  	int result;
	int val;
	int fd;
	struct termios new_term;
  	
	/* the port should have been specified */
	fd = open( init_port, O_RDWR | O_NONBLOCK );
	if( fd < 0 ) {
		printf( "Cannot open %s to connect to Initializer: %s\n", init_port, 
			strerror( errno ) );
		exit(-1);
		}

	if( isatty( fd ) == 0 ) {
		printf( "%s in NOT a tty device\n", init_port );
		exit(-1);
		}
#ifndef LINUX
	/* seek to the beginning of the "file" */
	if( lseek( fd, 0, SEEK_SET ) < 0 ) {
		printf( "Cannot seek to beginning of device: %s\n", strerror( errno ) );
		exit( -1 );
		}
#endif 

	/* save the terminal's characteristics to restore on exit */
	if( tcgetattr( fd, saved_term ) < 0 ) {
		printf( "tcgetattr error: %s\n", strerror( errno ) );
		exit( -1 );
		}

	/* read the current line discipline for later restoration */
	if( ioctl( fd, TIOCGETD, &saved_ldisc ) < 0 )
		printf( "Unable to get line discipline: %s\n", strerror( errno ) );

	/* specify an appropriate line discipline for this use */
	ldisc = TTYDISC;
	if( ioctl( fd, TIOCSETD, &ldisc ) < 0 )
		printf( "Unable to set line discipline: %s \n", strerror( errno ) );
	
	/* set the terminal port characteristics for the Initializer */
	new_term = *saved_term;
	new_term.c_cflag &= ~(CSIZE | PARENB | CRTS_IFLOW);
	new_term.c_cflag |= CS8 | CLOCAL | CREAD;
	new_term.c_oflag &= ~(OPOST);
	new_term.c_lflag = 0;
	new_term.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON | IXOFF);
	new_term.c_iflag |= IGNBRK | IGNCR;
	new_term.c_cc[VMIN] = 1;   /* 1 byte minimum */
	new_term.c_cc[VTIME] = 2; /* 2 second timeout */

	cfsetispeed( &new_term, B2400 );
	cfsetospeed( &new_term, B2400 );

	if( tcsetattr( fd, TCSAFLUSH, &new_term ) < 0 ) {
		printf( "tcsetattr error: %s\n", strerror( errno ) );
		exit( -1 );
		}
	
	/* clear the RTS and DTR lines */
	state = TIOCM_RTS | TIOCM_DTR;
	if( ioctl( fd, TIOCMBIC,  &state ) < 0 ) { 
		printf( "error clearing DTR and RTS: %s\n", strerror( errno ) );
		exit( -1 );
		}

	/* clear non-blocking */
	val = fcntl( fd, F_GETFL, 0);
	val &= ~O_NONBLOCK;
	fcntl( fd, F_SETFL, val );
	
	/* tell the caller what the file descriptor is for this port */
	*port_fd = fd;
	*sldisc = saved_ldisc;
	}
